@@ -12,7 +12,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'AST_C_DIR': 'Python',
  'AST_H': 'Include/Python-ast.h',
  'AST_H_DIR': 'Include',
- 'BASECFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code',
+ 'BASECFLAGS': '-Wno-unused-result -Wsign-compare',
  'BASECPPFLAGS': '',
  'BASEMODLIBS': '',
  'BINDIR': '/usr/bin',
@@ -26,11 +26,11 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BYTESTR_DEPS': '\\',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
- 'CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code -DNDEBUG -g '
-           '-fwrapv -O3 -Wall -Wstrict-prototypes -march=armv8-a -O2 -pipe '
+ 'CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
+           '-Wstrict-prototypes -march=armv8-a -O2 -pipe -fstack-protector '
+           '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
            '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a -O2 '
-           '-pipe -fstack-protector --param=ssp-buffer-size=4 -march=armv8-a '
-           '-O2 -pipe -fstack-protector --param=ssp-buffer-size=4',
+           '-pipe -fstack-protector --param=ssp-buffer-size=4',
  'CFLAGSFORSHARED': '-fPIC',
  'CFLAGS_NODIST': '',
  'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in '
@@ -52,8 +52,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.5m',
  'COREPYTHONPATH': ':plat-linux',
- 'COVERAGE_INFO': '/build/python/src/Python-3.5.1/coverage.info',
- 'COVERAGE_REPORT': '/build/python/src/Python-3.5.1/lcov-report',
+ 'COVERAGE_INFO': '/build/python/src/Python-3.5.2/coverage.info',
+ 'COVERAGE_REPORT': '/build/python/src/Python-3.5.2/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I./Include -D_FORTIFY_SOURCE=2 '
@@ -208,7 +208,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_GETPRIORITY': 1,
  'HAVE_GETPWENT': 1,
  'HAVE_GETRANDOM': 0,
- 'HAVE_GETRANDOM_SYSCALL': 0,
+ 'HAVE_GETRANDOM_SYSCALL': 1,
  'HAVE_GETRESGID': 1,
  'HAVE_GETRESUID': 1,
  'HAVE_GETSID': 1,
@@ -255,6 +255,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_LINUX_CAN_RAW_FD_FRAMES': 1,
  'HAVE_LINUX_CAN_RAW_H': 1,
  'HAVE_LINUX_NETLINK_H': 1,
+ 'HAVE_LINUX_RANDOM_H': 1,
  'HAVE_LINUX_TIPC_H': 1,
  'HAVE_LOCKF': 1,
  'HAVE_LOG1P': 1,
@@ -319,6 +320,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_RL_COMPLETION_MATCHES': 1,
  'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 1,
  'HAVE_RL_PRE_INPUT_HOOK': 1,
+ 'HAVE_RL_RESIZE_TERMINAL': 1,
  'HAVE_ROUND': 1,
  'HAVE_SCHED_GET_PRIORITY_MAX': 1,
  'HAVE_SCHED_H': 1,
@@ -385,7 +387,6 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_STRUCT_STAT_ST_GEN': 0,
  'HAVE_STRUCT_STAT_ST_RDEV': 1,
  'HAVE_STRUCT_TM_TM_ZONE': 1,
- 'HAVE_ST_BLOCKS': 1,
  'HAVE_SYMLINK': 1,
  'HAVE_SYMLINKAT': 1,
  'HAVE_SYNC': 1,
@@ -573,7 +574,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PLATDIR': 'plat-linux',
  'POBJS': '\\',
  'POSIX_SEMAPHORES_NOT_ENABLED': 0,
- 'PROFILE_TASK': '-m test.regrtest --pgo',
+ 'PROFILE_TASK': '-m test.regrtest --pgo -x test_asyncore test_gdb '
+                 'test_multiprocessing_fork test_multiprocessing_forkserver '
+                 'test_multiprocessing_main_handling '
+                 'test_multiprocessing_spawn test_subprocess',
  'PSRCS': '\\',
  'PTHREAD_SYSTEM_SCHED_SUPPORTED': 1,
  'PURIFY': '',
@@ -588,19 +592,18 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHON_FOR_BUILD': './python -E',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
- 'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code -DNDEBUG '
-              '-g -fwrapv -O3 -Wall -Wstrict-prototypes -march=armv8-a -O2 '
-              '-pipe -fstack-protector --param=ssp-buffer-size=4 '
-              '-march=armv8-a -O2 -pipe -fstack-protector '
+ 'PY_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 -Wall '
+              '-Wstrict-prototypes -march=armv8-a -O2 -pipe -fstack-protector '
               '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
-              '-fstack-protector --param=ssp-buffer-size=4',
+              '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a -O2 '
+              '-pipe -fstack-protector --param=ssp-buffer-size=4',
  'PY_CFLAGS_NODIST': '-Werror=declaration-after-statement',
- 'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -Wunreachable-code '
-                   '-DNDEBUG -g -fwrapv -O3 -Wall -Wstrict-prototypes '
-                   '-march=armv8-a -O2 -pipe -fstack-protector '
-                   '--param=ssp-buffer-size=4 -march=armv8-a -O2 -pipe '
+ 'PY_CORE_CFLAGS': '-Wno-unused-result -Wsign-compare -DNDEBUG -g -fwrapv -O3 '
+                   '-Wall -Wstrict-prototypes -march=armv8-a -O2 -pipe '
                    '-fstack-protector --param=ssp-buffer-size=4 -march=armv8-a '
                    '-O2 -pipe -fstack-protector --param=ssp-buffer-size=4 '
+                   '-march=armv8-a -O2 -pipe -fstack-protector '
+                   '--param=ssp-buffer-size=4 '
                    '-Werror=declaration-after-statement -I. -IInclude '
                    '-I./Include -D_FORTIFY_SOURCE=2 -D_FORTIFY_SOURCE=2 -fPIC '
                    '-DPy_BUILD_CORE',
@@ -618,7 +621,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.1',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.2',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -660,9 +663,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.1 ./python',
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.2 ./python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.1 ./python '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python/src/Python-3.5.2 ./python '
                './Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -687,8 +690,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/python/src/Python-3.5.1',
- 'abs_srcdir': '/build/python/src/Python-3.5.1',
+ 'abs_builddir': '/build/python/src/Python-3.5.2',
+ 'abs_srcdir': '/build/python/src/Python-3.5.2',
+ 'cross_compiling': 'no',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
